/*****************************************************************************
 *
 * wave.h
 *
 * PHASEX:  [P]hase [H]armonic [A]dvanced [S]ynthesis [EX]periment
 *
 * Copyright (C) 1999-2007 William Weston <weston@sysex.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************************/
#ifndef _PHASEX_WAVE_H_
#define _PHASEX_WAVE_H_

#include "phasex.h"


/* hardcoded waveform numbers (should match the functions) */
#define WAVE_SINE	0
#define WAVE_TRIANGLE	1
#define WAVE_SAW	2
#define WAVE_REVSAW	3
#define WAVE_SQUARE	4
#define WAVE_STAIR	5
#define WAVE_SAW_S	6
#define WAVE_REVSAW_S	7
#define WAVE_SQUARE_S	8
#define WAVE_STAIR_S	9
#define WAVE_IDENTITY	10
#define WAVE_NULL	11
#define WAVE_12		12
#define WAVE_13		13
#define WAVE_14		14
#define WAVE_15		15
#define WAVE_16		16
#define WAVE_17		17
#define WAVE_18		18
#define WAVE_19		19
#define WAVE_20		20
#define WAVE_21		21
#define WAVE_22		22
#define WAVE_23		23
#define WAVE_24		24
#define WAVE_25		25
#define WAVE_26		26
#define WAVE_27		27
#define WAVE_28		28
#define WAVE_29		29
#define WAVE_30		30
#define WAVE_31		31


/* lookup macro for exponential frequency scaling */
#define halfsteps_to_freq_mult(val) (freq_shift_table[(int)(((val)*TUNING_RESOLUTION)+FREQ_SHIFT_ZERO_OFFSET)])


/* lookup tables */
extern float_t freq_shift_table[FREQ_SHIFT_TABLE_SIZE];
extern float_t wave_table[NUM_WAVEFORMS][WAVEFORM_SIZE];
extern float_t freq_table[128][648];
extern float_t keyfollow_table[128][128];
extern float_t mix_table[128];
extern float_t gain_table[128];
extern float_t env_curve[ENV_CURVE_SIZE+1];
extern int env_table[128];

/* global base tuning frequency */
extern double a4freq;


void build_freq_table(void);
void build_freq_shift_table(void);
void build_waveform_tables(void);
void build_env_tables(void);
void build_mix_table(void);
void build_gain_table(void);
void build_keyfollow_table(void);


/* massaging functions */
//void massage_waves(void);

/* these are the functions for building the initial waveforms */
/* these should _only_ be used in the startup code */
float_t func_square(unsigned int sample_num);
float_t func_saw(unsigned int sample_num);
float_t func_revsaw(unsigned int sample_num);
float_t func_saw_s(unsigned int sample_num);
float_t func_revsaw_s(unsigned int sample_num);
float_t func_stair(unsigned int sample_num);
float_t func_triangle(unsigned int sample_num);


#endif /* _PHASEX_WAVE_H_ */
